use std::{env, net::SocketAddrV4};

use askama::Template;
use warp::{http::StatusCode, reject::Rejection, reply::{html, json, with_status, Reply}, Filter};


#[derive(Debug, Clone, Template)]
#[template(path = "invite.html")]
struct Invite {
    pub name: String,
    pub invitee: String
}

async fn gen(name: String, invitee: String) -> Result<Box<dyn Reply>, Rejection> {
    let inv = Invite { name, invitee };
    let rendered = inv.render();
    if let Err(err) = rendered {
        Ok(
            Box::new(
                with_status(json(&err.to_string()), StatusCode::INTERNAL_SERVER_ERROR)
            )
        )
    } else {
        Ok(Box::new(html(rendered.unwrap())))
    }
}

#[tokio::main]
async fn main() {
    let route = warp::path!("gen" / String / String).and_then(gen);
    let url = env::var("LISTEN").unwrap_or("0.0.0.0:80".into()).parse::<SocketAddrV4>().expect("Invalid LISTEN url");
    println!("Listening on {url}");
    warp::serve(route).bind(url).await;
}
